/*
 * Copyright 2009 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.kantega.jexmec;

import java.io.File;
import java.io.FileFilter;
import java.util.*;

/**
 */
public class GroovyClassLoaderStrategy implements ClassLoaderStrategy {
    private File groovyScriptDirectory;

    private final List<ClassLoaderListener> classLoaderListeners = Collections.synchronizedList(new ArrayList<ClassLoaderListener>());
    private Set<String> excludedDirectories = new HashSet<String>();


    public GroovyClassLoaderStrategy(File groovyScriptDirectory) {
        this.groovyScriptDirectory = groovyScriptDirectory;
        excludedDirectories.add(".svn");
    }

    public void addClassLoaderListener(ClassLoaderListener listener) {
        classLoaderListeners.add(listener);
    }

    public void removeClassLoaderListener(ClassLoaderListener listener) {
        classLoaderListeners.remove(listener);
    }

    public void start() {
        if(groovyScriptDirectory != null && groovyScriptDirectory.exists() && groovyScriptDirectory.isDirectory()) {

            final File[] directories = getDirectories();

            for(File directory : directories) {
                JexmecGroovyClassLoader loader = new JexmecGroovyClassLoader(directory);
                fireClassLoaderAdded(loader);
            }
        }
    }

    private void fireClassLoaderAdded(ClassLoader loader) {
        for (ClassLoaderListener listener : classLoaderListeners) {
            listener.classLoaderAdded(loader);
        }
    }


    private File[] getDirectories() {
        return groovyScriptDirectory.listFiles(new FileFilter() {
            public boolean accept(File file) {
                return file.isDirectory() && !excludedDirectories.contains(file.getName());
            }
        });
    }

    public void stop() {
    }
}
