/*
 * Copyright 2009 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.kantega.jexmec;

import groovy.lang.GroovyClassLoader;

import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
import java.io.File;
import java.io.FileFilter;
import java.io.IOException;

/**
 */
public class GroovyPluginLoader<P extends Plugin> implements PluginLoader<P> {

    public List<P> loadPlugins(Class<P> pluginClass, ClassLoader classLoader, ServiceLocator serviceLocator) {
        if (classLoader instanceof JexmecGroovyClassLoader) {
            List<P> plugins = new ArrayList<P>();
            JexmecGroovyClassLoader loader = (JexmecGroovyClassLoader) classLoader;

            final File[] scripts = getScripts(loader.getDirectory());

            for (File script : scripts) {
                GroovyClassLoader groovyClassLoader = new GroovyClassLoader(loader);
                try {

                    @SuppressWarnings("unchecked")
                    Class<P> groovyClass = groovyClassLoader.parseClass(script);

                    P plugin = (P) groovyClass.newInstance();

                    plugins.add(plugin);

                } catch (IOException e) {
                    throw new RuntimeException(e);
                } catch (IllegalAccessException e) {
                    throw new RuntimeException(e);
                } catch (InstantiationException e) {
                    throw new RuntimeException(e);
                }
            }
            return plugins;
        } else {
            return Collections.emptyList();
        }

    }

    public void unloadPlugins(ClassLoader classLoader) {
    }

    private File[] getScripts(File directory) {
        return directory.listFiles(new FileFilter() {
            public boolean accept(File file) {
                return file.isFile() && file.getName().toLowerCase().endsWith(".groovy");
            }
        });
    }

    public void start() {
    }

    public void stop() {
    }
}
